@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-only
 /* DVB USB framework compliant Linux driver for the
  *	DVBWorld DVB-S 2101, 2102, DVB-S2 2104, DVB-C 3101,
  *	TeVii S421, S480, S482, S600, S630, S632, S650, S660, S662,
@@ -7,10 +8,6 @@
  *	Terratec Cinergy S2 cards
  * Copyright (C) 2008-2012 Igor M. Liplianin (liplianin@me.by)
  *
- *	This program is free software; you can redistribute it and/or modify it
- *	under the terms of the GNU General Public License as published by the
- *	Free Software Foundation, version 2.
- *
  * see Documentation/media/dvb-drivers/dvb-usb.rst for more information
  */
 #include <media/dvb-usb-ids.h>
@@ -131,10 +128,6 @@ static int dw2102_i2c_transfer(struct i2c_adapter *adap, struct i2c_msg msg[],
 
 	switch (num) {
 	case 2:
-		if (msg[0].len < 1) {
-			num = -EOPNOTSUPP;
-			break;
-		}
 		/* read stv0299 register */
 		value = msg[0].buf[0];/* register */
 		for (i = 0; i < msg[1].len; i++) {
@@ -146,10 +139,6 @@ static int dw2102_i2c_transfer(struct i2c_adapter *adap, struct i2c_msg msg[],
 	case 1:
 		switch (msg[0].addr) {
 		case 0x68:
-			if (msg[0].len < 2) {
-				num = -EOPNOTSUPP;
-				break;
-			}
 			/* write to stv0299 register */
 			buf6[0] = 0x2a;
 			buf6[1] = msg[0].buf[0];
@@ -159,10 +148,6 @@ static int dw2102_i2c_transfer(struct i2c_adapter *adap, struct i2c_msg msg[],
 			break;
 		case 0x60:
 			if (msg[0].flags == 0) {
-				if (msg[0].len < 4) {
-					num = -EOPNOTSUPP;
-					break;
-				}
 			/* write to tuner pll */
 				buf6[0] = 0x2c;
 				buf6[1] = 5;
@@ -174,10 +159,6 @@ static int dw2102_i2c_transfer(struct i2c_adapter *adap, struct i2c_msg msg[],
 				dw210x_op_rw(d->udev, 0xb2, 0, 0,
 						buf6, 7, DW210X_WRITE_MSG);
 			} else {
-				if (msg[0].len < 1) {
-					num = -EOPNOTSUPP;
-					break;
-				}
 			/* read from tuner */
 				dw210x_op_rw(d->udev, 0xb5, 0, 0,
 						buf6, 1, DW210X_READ_MSG);
@@ -185,20 +166,12 @@ static int dw2102_i2c_transfer(struct i2c_adapter *adap, struct i2c_msg msg[],
 			}
 			break;
 		case (DW2102_RC_QUERY):
-			if (msg[0].len < 2) {
-				num = -EOPNOTSUPP;
-				break;
-			}
 			dw210x_op_rw(d->udev, 0xb8, 0, 0,
 					buf6, 2, DW210X_READ_MSG);
 			msg[0].buf[0] = buf6[0];
 			msg[0].buf[1] = buf6[1];
 			break;
 		case (DW2102_VOLTAGE_CTRL):
-			if (msg[0].len < 1) {
-				num = -EOPNOTSUPP;
-				break;
-			}
 			buf6[0] = 0x30;
 			buf6[1] = msg[0].buf[0];
 			dw210x_op_rw(d->udev, 0xb2, 0, 0,
@@ -719,7 +692,6 @@ static int su3000_i2c_transfer(struct i2c_adapter *adap, struct i2c_msg msg[],
 {
 	struct dvb_usb_device *d = i2c_get_adapdata(adap);
 	struct dw2102_state *state;
-	int j;
 
 	if (!d)
 		return -ENODEV;
@@ -733,11 +705,11 @@ static int su3000_i2c_transfer(struct i2c_adapter *adap, struct i2c_msg msg[],
 		return -EAGAIN;
 	}
 
-	j = 0;
-	while (j < num) {
-		switch (msg[j].addr) {
+	switch (num) {
+	case 1:
+		switch (msg[0].addr) {
 		case SU3000_STREAM_CTRL:
-			state->data[0] = msg[j].buf[0] + 0x36;
+			state->data[0] = msg[0].buf[0] + 0x36;
 			state->data[1] = 3;
 			state->data[2] = 0;
 			if (dvb_usb_generic_rw(d, state->data, 3,
@@ -749,86 +721,61 @@ static int su3000_i2c_transfer(struct i2c_adapter *adap, struct i2c_msg msg[],
 			if (dvb_usb_generic_rw(d, state->data, 1,
 					state->data, 2, 0) < 0)
 				err("i2c transfer failed.");
-			msg[j].buf[1] = state->data[0];
-			msg[j].buf[0] = state->data[1];
+			msg[0].buf[1] = state->data[0];
+			msg[0].buf[0] = state->data[1];
 			break;
 		default:
-			/* if the current write msg is followed by a another
-			 * read msg to/from the same address
-			 */
-			if ((j+1 < num) && (msg[j+1].flags & I2C_M_RD) &&
-			    (msg[j].addr == msg[j+1].addr)) {
-				/* join both i2c msgs to one usb read command */
-				if (4 + msg[j].len > sizeof(state->data)) {
-					warn("i2c combined wr/rd: write len=%d is too big!\n",
-					    msg[j].len);
-					num = -EOPNOTSUPP;
-					break;
-				}
-				if (1 + msg[j+1].len > sizeof(state->data)) {
-					warn("i2c combined wr/rd: read len=%d is too big!\n",
-					    msg[j+1].len);
-					num = -EOPNOTSUPP;
-					break;
-				}
-
-				state->data[0] = 0x09;
-				state->data[1] = msg[j].len;
-				state->data[2] = msg[j+1].len;
-				state->data[3] = msg[j].addr;
-				memcpy(&state->data[4], msg[j].buf, msg[j].len);
-
-				if (dvb_usb_generic_rw(d, state->data, msg[j].len + 4,
-					state->data, msg[j+1].len + 1, 0) < 0)
-					err("i2c transfer failed.");
-
-				memcpy(msg[j+1].buf, &state->data[1], msg[j+1].len);
-				j++;
-				break;
-			}
-
-			if (msg[j].flags & I2C_M_RD) {
-				/* single read */
-				if (4 + msg[j].len > sizeof(state->data)) {
-					warn("i2c rd: len=%d is too big!\n", msg[j].len);
-					num = -EOPNOTSUPP;
-					break;
-				}
-
-				state->data[0] = 0x09;
-				state->data[1] = 0;
-				state->data[2] = msg[j].len;
-				state->data[3] = msg[j].addr;
-				memcpy(&state->data[4], msg[j].buf, msg[j].len);
-
-				if (dvb_usb_generic_rw(d, state->data, 4,
-					state->data, msg[j].len + 1, 0) < 0)
-					err("i2c transfer failed.");
-
-				memcpy(msg[j].buf, &state->data[1], msg[j].len);
-				break;
-			}
-
-			/* single write */
-			if (3 + msg[j].len > sizeof(state->data)) {
-				warn("i2c wr: len=%d is too big!\n", msg[j].len);
+			if (3 + msg[0].len > sizeof(state->data)) {
+				warn("i2c wr: len=%d is too big!\n",
+				     msg[0].len);
 				num = -EOPNOTSUPP;
 				break;
 			}
 
+			/* always i2c write*/
 			state->data[0] = 0x08;
-			state->data[1] = msg[j].addr;
-			state->data[2] = msg[j].len;
+			state->data[1] = msg[0].addr;
+			state->data[2] = msg[0].len;
 
-			memcpy(&state->data[3], msg[j].buf, msg[j].len);
+			memcpy(&state->data[3], msg[0].buf, msg[0].len);
 
-			if (dvb_usb_generic_rw(d, state->data, msg[j].len + 3,
+			if (dvb_usb_generic_rw(d, state->data, msg[0].len + 3,
 						state->data, 1, 0) < 0)
 				err("i2c transfer failed.");
-		} // switch
-		j++;
 
-	} // while
+		}
+		break;
+	case 2:
+		/* always i2c read */
+		if (4 + msg[0].len > sizeof(state->data)) {
+			warn("i2c rd: len=%d is too big!\n",
+			     msg[0].len);
+			num = -EOPNOTSUPP;
+			break;
+		}
+		if (1 + msg[1].len > sizeof(state->data)) {
+			warn("i2c rd: len=%d is too big!\n",
+			     msg[1].len);
+			num = -EOPNOTSUPP;
+			break;
+		}
+
+		state->data[0] = 0x09;
+		state->data[1] = msg[0].len;
+		state->data[2] = msg[1].len;
+		state->data[3] = msg[0].addr;
+		memcpy(&state->data[4], msg[0].buf, msg[0].len);
+
+		if (dvb_usb_generic_rw(d, state->data, msg[0].len + 4,
+					state->data, msg[1].len + 1, 0) < 0)
+			err("i2c transfer failed.");
+
+		memcpy(msg[1].buf, &state->data[1], msg[1].len);
+		break;
+	default:
+		warn("more than 2 i2c messages at a time is not handled yet.");
+		break;
+	}
 	mutex_unlock(&d->data_mutex);
 	mutex_unlock(&d->i2c_mutex);
 	return num;
@@ -1639,7 +1586,7 @@ static int tt_s2_4600_frontend_attach(struct dvb_usb_adapter *adap)
 	m88ds3103_pdata.lnb_hv_pol = 1;
 	m88ds3103_pdata.lnb_en_pol = 0;
 	memset(&board_info, 0, sizeof(board_info));
-	strlcpy(board_info.type, "m88ds3103", I2C_NAME_SIZE);
+	strscpy(board_info.type, "m88ds3103", I2C_NAME_SIZE);
 	board_info.addr = 0x68;
 	board_info.platform_data = &m88ds3103_pdata;
 	request_module("m88ds3103");
@@ -1658,7 +1605,7 @@ static int tt_s2_4600_frontend_attach(struct dvb_usb_adapter *adap)
 	/* attach tuner */
 	ts2020_config.fe = adap->fe_adap[0].fe;
 	memset(&board_info, 0, sizeof(board_info));
-	strlcpy(board_info.type, "ts2022", I2C_NAME_SIZE);
+	strscpy(board_info.type, "ts2022", I2C_NAME_SIZE);
 	board_info.addr = 0x60;
 	board_info.platform_data = &ts2020_config;
 	request_module("ts2020");
